@@ -4,6 +4,9 @@
 //! terms of the trait [`crate::SOP`], hence it can be shared between
 //! SOP implementations.
 //!
+//! Via the "cliv" feature, it can also produce a smaller standalone
+//! binary that only implements the sopv subset.
+//!
 //! To use, add this snippet to `Cargo.toml`:
 //!
 //! ```toml
@@ -11,11 +14,17 @@
 //! path = "src/main.rs"
 //! required-features = ["cli"]
 //!
+//! [[bin]]
+//! path = "src/mainv.rs"
+//! required-features = ["cliv"]
+//!
 //! [features]
 //! cli = ["sop/cli"]
+//! cliv = ["sop/cliv"]
 //! ```
 //!
-//! And create `src/main.rs` along the lines of:
+//! And create `src/main.rs` and `src/mainv.rs`, both containing
+//! something along the lines of:
 //!
 //! ```rust,ignore
 //! fn main() {
@@ -23,6 +32,10 @@
 //! }
 //! ```
 //!
+//! Note: If you don't need to tweak your implementation for the
+//! verification subset of SOP, you can also build both binaries from
+//! the same source.
+//!
 //! ## Generating shell completions
 //!
 //! To create shell completions, add this snippet to `Cargo.toml`:
@@ -35,16 +48,16 @@
 //! And create `build.rs` along the lines of:
 //!
 //! ```rust,no_run
-//! #[cfg(feature = "cli")]
 //! fn main() {
+//!     #[allow(dead_code)]
 //!     let outdir = std::env::var_os("CARGO_TARGET_DIR")
 //!         .or(std::env::var_os("OUT_DIR"))
 //!         .expect("cargo to set OUT_DIR");
-//!     sop::cli::write_shell_completions("sqop", outdir).unwrap();
+//!     #[cfg(feature = "cli")]
+//!     sop::cli::write_shell_completions("sqop", &outdir).unwrap();
+//!     #[cfg(feature = "cliv")]
+//!     sop::cli::write_shell_completions("sqopv", &outdir).unwrap();
 //! }
-//!
-//! #[cfg(not(feature = "cli"))]
-//! fn main() {}
 //! ```
 //!
 //! # Features and limitations
@@ -65,14 +78,18 @@ use chrono::{DateTime, NaiveTime, offset::Utc};
 use clap::{CommandFactory, FromArgMatches, Parser};
 
 use crate::{
-    ArmorLabel,
-    EncryptAs,
     Save,
     Load,
+    SOP,
+};
+
+#[cfg(feature = "cli")]
+use crate::{
+    ArmorLabel,
+    EncryptAs,
     InlineSignAs,
     Password,
     SignAs,
-    SOP,
 };
 
 #[derive(Debug, Parser)]
@@ -84,6 +101,7 @@ enum Operation {
     /// Invoked without arguments, returns name and version of the SOP
     /// implementation.
     #[clap(display_order = 100)]
+    #[cfg(any(feature = "cliv", feature = "cli"))]
     Version {
         /// Returns name and version of the primary underlying OpenPGP
         /// toolkit.
@@ -109,11 +127,13 @@ enum Operation {
     /// Emits a list of profiles supported by the identified
     /// subcommand.
     #[clap(display_order = 150)]
+    #[cfg(feature = "cli")]
     ListProfiles {
         subcommand: String,
     },
     /// Generates a Secret Key.
     #[clap(display_order = 200)]
+    #[cfg(feature = "cli")]
     GenerateKey {
         /// Don't ASCII-armor output.
         #[clap(long)]
@@ -148,6 +168,7 @@ enum Operation {
     /// any of the supplied `--old-key-password` arguments, this
     /// subcommand should fail with `KEY_IS_PROTECTED`.
     #[clap(display_order = 210)]
+    #[cfg(feature = "cli")]
     ChangeKeyPassword {
         /// Don't ASCII-armor output.
         #[clap(long)]
@@ -166,6 +187,7 @@ enum Operation {
     /// Generate a revocation certificate for each Transferable Secret
     /// Key found.
     #[clap(display_order = 220)]
+    #[cfg(feature = "cli")]
     RevokeKey {
         /// Don't ASCII-armor output.
         #[clap(long)]
@@ -177,6 +199,7 @@ enum Operation {
 
     /// Extracts a Certificate from a Secret Key.
     #[clap(display_order = 300)]
+    #[cfg(feature = "cli")]
     ExtractCert {
         /// Don't ASCII-armor output.
         #[clap(long)]
@@ -184,6 +207,7 @@ enum Operation {
     },
     /// Creates Detached Signatures.
     #[clap(display_order = 400)]
+    #[cfg(feature = "cli")]
     Sign {
         /// Don't ASCII-armor output.
         #[clap(long)]
@@ -202,6 +226,7 @@ enum Operation {
     },
     /// Verifies Detached Signatures.
     #[clap(display_order = 500)]
+    #[cfg(any(feature = "cli", feature = "cliv"))]
     Verify {
         /// Consider signatures before this date invalid.
         #[clap(long)]
@@ -216,6 +241,7 @@ enum Operation {
     },
     /// Encrypts a Message.
     #[clap(display_order = 600)]
+    #[cfg(feature = "cli")]
     Encrypt {
         /// Don't ASCII-armor output.
         #[clap(long)]
@@ -243,6 +269,7 @@ enum Operation {
     },
     /// Decrypts a Message.
     #[clap(display_order = 700)]
+    #[cfg(feature = "cli")]
     Decrypt {
         /// Write the session key here.
         #[clap(long)]
@@ -273,6 +300,7 @@ enum Operation {
     },
     /// Converts binary OpenPGP data to ASCII.
     #[clap(display_order = 800)]
+    #[cfg(feature = "cli")]
     Armor {
         /// Indicates the kind of data.
         #[clap(long, default_value = "auto", hide(true))]
@@ -280,10 +308,12 @@ enum Operation {
     },
     /// Converts ASCII OpenPGP data to binary.
     #[clap(display_order = 900)]
+    #[cfg(feature = "cli")]
     Dearmor {
     },
     /// Splits Signatures from an Inline-Signed Message.
     #[clap(display_order = 750)]
+    #[cfg(feature = "cli")]
     InlineDetach {
         /// Don't ASCII-armor the signatures.
         #[clap(long)]
@@ -294,6 +324,7 @@ enum Operation {
     },
     /// Verifies Inline-Signed Messages.
     #[clap(display_order = 751)]
+    #[cfg(any(feature = "cli", feature = "cliv"))]
     InlineVerify {
         /// Consider signatures before this date invalid.
         #[clap(long)]
@@ -309,6 +340,7 @@ enum Operation {
     },
     /// Creates Inline-Signed Messages.
     #[clap(display_order = 752)]
+    #[cfg(feature = "cli")]
     InlineSign {
         /// Don't ASCII-armor output.
         #[clap(long)]
@@ -444,6 +476,7 @@ where
             }
         },
 
+        #[cfg(feature = "cli")]
         Operation::ListProfiles { subcommand, } => {
             let profiles = match subcommand.as_str() {
                 "generate-key" => sop.generate_key()?.list_profiles(),
@@ -455,6 +488,7 @@ where
             }
         },
 
+        #[cfg(feature = "cli")]
         Operation::GenerateKey {
             no_armor,
             profile,
@@ -479,6 +513,7 @@ where
                 .to_writer(! no_armor, &mut io::stdout())?;
         },
 
+        #[cfg(feature = "cli")]
         Operation::ChangeKeyPassword {
             no_armor,
             new_key_password,
@@ -495,6 +530,7 @@ where
                 .to_writer(! no_armor, &mut io::stdout())?;
         }
 
+        #[cfg(feature = "cli")]
         Operation::RevokeKey { no_armor, with_key_password, } => {
             let mut op = sop.revoke_key()?;
             for p in with_key_password {
@@ -504,12 +540,14 @@ where
                 .to_writer(! no_armor, &mut io::stdout())?;
         }
 
+        #[cfg(feature = "cli")]
         Operation::ExtractCert { no_armor, } => {
             sop.extract_cert()?
                 .keys(&K::from_reader(sop, &mut io::stdin())?)?
                 .to_writer(! no_armor, &mut io::stdout())?;
         },
 
+        #[cfg(feature = "cli")]
         Operation::Sign {
             no_armor,
             as_,
@@ -532,6 +570,7 @@ where
             sigs.to_writer(! no_armor, &mut io::stdout())?;
         },
 
+        #[cfg(any(feature = "cli", feature = "cliv"))]
         Operation::Verify { not_before, not_after, signatures, certs, } => {
             let mut op = sop.verify()?;
             if let Some(t) = not_before {
@@ -552,6 +591,7 @@ where
             }
         },
 
+        #[cfg(feature = "cli")]
         Operation::Encrypt {
             no_armor,
             profile,
@@ -601,6 +641,7 @@ where
             }
         },
 
+        #[cfg(feature = "cli")]
         Operation::Decrypt {
             session_key_out,
             with_session_key,
@@ -679,6 +720,7 @@ where
             }
         },
 
+        #[cfg(feature = "cli")]
         Operation::Armor { label, } => {
             #[allow(deprecated)]
             let op = sop.armor()?.label(label);
@@ -686,12 +728,14 @@ where
                 .to_writer(&mut io::stdout())?;
         },
 
+        #[cfg(feature = "cli")]
         Operation::Dearmor {} => {
             let op = sop.dearmor()?;
             op.data(&mut io::stdin())?
                 .to_writer(&mut io::stdout())?;
         },
 
+        #[cfg(feature = "cli")]
         Operation::InlineDetach { no_armor, signatures_out, } => {
             let op = sop.inline_detach()?;
             let mut signatures_out = create_file(signatures_out)?;
@@ -700,6 +744,7 @@ where
             signatures.to_writer(! no_armor, &mut signatures_out)?;
         },
 
+        #[cfg(any(feature = "cli", feature = "cliv"))]
         Operation::InlineVerify {
             not_before,
             not_after,
@@ -731,6 +776,7 @@ where
             }
         },
 
+        #[cfg(feature = "cli")]
         Operation::InlineSign { no_armor, as_, with_key_password, keys, } => {
             let mut op = sop.inline_sign()?.mode(as_);
             if no_armor {
@@ -822,6 +868,7 @@ fn load_file<S: AsRef<str>>(file: S)
 ///
 /// Passwords are indirect data types, meaning that they are
 /// loaded from files or special designators.
+#[cfg(feature = "cli")]
 fn get_password<S: AsRef<str>>(location: S) -> Result<Password> {
     let mut stream = load_file(location)?;
     let mut pw = Vec::new();
@@ -835,6 +882,7 @@ fn get_password<S: AsRef<str>>(location: S) -> Result<Password> {
 ///
 /// Passwords are indirect data types, meaning that they are
 /// loaded from files or special designators.
+#[cfg(feature = "cli")]
 fn get_password_unchecked<S: AsRef<str>>(location: S) -> Result<Password> {
     let mut stream = load_file(location)?;
     let mut pw = Vec::new();
