@@ -83,7 +83,7 @@ pub mod errors;
 use errors::*;
 pub mod plumbing;
 
-#[cfg(feature = "cli")]
+#[cfg(any(feature = "cli", feature = "cliv"))]
 pub mod cli;
 
 /// Loads objects like certs and keys.
