@@ -367,7 +367,7 @@ pub struct Verification {
     message: Option<String>,
 }
 
-#[cfg(feature = "cli")]
+#[cfg(any(feature = "cli", feature = "cliv"))]
 impl fmt::Display for Verification {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f,
@@ -449,7 +449,7 @@ pub enum SignatureMode {
     Binary,
 }
 
-#[cfg(feature = "cli")]
+#[cfg(any(feature = "cli", feature = "cliv"))]
 impl fmt::Display for SignatureMode {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
